package cn.timer.api.controller.jxgl;

import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.timer.api.bean.jxgl.JxglAppraisalIndicatorsT;
import cn.timer.api.bean.jxgl.JxglAppraisalItemT;
import cn.timer.api.bean.jxgl.JxglAppraisalT;
import cn.timer.api.bean.jxgl.JxglBasicSetting;
import cn.timer.api.bean.jxgl.JxglPerformanceAppraisal;
import cn.timer.api.bean.jxgl.JxglPerformanceRating;
import cn.timer.api.bean.qyzx.QyzxEmpEntAsso;
import cn.timer.api.config.annotation.CurrentUser;
import cn.timer.api.config.annotation.UserBean;
import cn.timer.api.config.enuminterface.JxglEnumInterface.BeingAppraisalSts;
import cn.timer.api.config.enuminterface.JxglEnumInterface.BeingAppraisalType;
import cn.timer.api.config.enums.CommonEnum;
import cn.timer.api.dao.jxgl.JxglAppraisalIndicatorsTMapper;
import cn.timer.api.dao.jxgl.JxglAppraisalItemTMapper;
import cn.timer.api.dao.jxgl.JxglAppraisalTMapper;
import cn.timer.api.dao.jxgl.JxglBasicSettingMapper;
import cn.timer.api.dao.jxgl.JxglPerformanceRatingMapper;
import cn.timer.api.dao.qyzx.QyzxEmpEntAssoMapper;
import cn.timer.api.dto.jxgl.BeingAppraisalPerson;
import cn.timer.api.dto.jxgl.PerformanceAppraisalDto;
import cn.timer.api.dto.jxgl.ProcessNode;
import cn.timer.api.utils.Result;
import cn.timer.api.utils.ResultUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@Api(tags = "3.1绩效管理")
@RestController
@Transactional(rollbackFor = Exception.class)
@RequestMapping(value = "/jxgl",  
produces = { "application/json","multipart/form-data" })
public class JxglController {

	@Autowired
	private QyzxEmpEntAssoMapper qyzxEmpEntAssoMapper;
	
	@Autowired
	private JxglBasicSettingMapper jxglBasicSettingMapper;
	
	@Autowired
	private JxglPerformanceRatingMapper jxglPerformanceRatingMapper;
	
	@Autowired
	private JxglAppraisalTMapper jxglAppraisalTMapper;
	
	@Autowired
	private JxglAppraisalIndicatorsTMapper jxglAppraisalIndicatorsTMapper;
	
	@Autowired
	private JxglAppraisalItemTMapper jsxglAppraisalItemTMapper;
	
	//TODO 基础设置
	/**
	 * 新增或编辑-基础设置
	 */
	@PostMapping(value = "/save_basic_setting")
	@ApiOperation(value = "1.新增或编辑-基础设置", httpMethod = "POST", notes = "新增或编辑-审批模板组")
	@ApiOperationSupport(order = 1)
	public Result<Object> saveAtg(@CurrentUser UserBean userBean ,@Validated @RequestBody JxglBasicSetting basicSetting) throws Exception{
		Integer orgCode = userBean.getOrgCode();
		Integer count = new LambdaQueryChainWrapper<QyzxEmpEntAsso>(qyzxEmpEntAssoMapper)
				.eq(QyzxEmpEntAsso::getEmpNum, userBean.getEmpNum())
				.in(QyzxEmpEntAsso::getUserType, CommonEnum.U_TYPE_ADMIN.getType(),CommonEnum.U_TYPE_C_ADMIN.getType())
				.count();
		
		if (count <= 0) {
			return ResultUtil.error("无权限编辑");
		}
		
		List<JxglPerformanceRating> listPr = basicSetting.getPerformanceRatings();
		if (listPr != null) {
			for (JxglPerformanceRating pr : listPr) {
				if (pr.getSectionMaxScore() > basicSetting.getMaxScore()) {
					return ResultUtil.error("区间值不能超过 最高分");
				}else if (pr.getSectionMinScore() >= pr.getSectionMaxScore()) {
					return ResultUtil.error("区间值 设置不正确");
				}
			}
		}
		JxglBasicSetting bS = jxglBasicSettingMapper.selectOne(new QueryWrapper<JxglBasicSetting>().lambda()
				.select(JxglBasicSetting::getId)
				.eq(JxglBasicSetting::getOrgCode, orgCode));
		if (bS != null) {
			// 删除 绩效等级
			jxglPerformanceRatingMapper.delete(new QueryWrapper<JxglPerformanceRating>().lambda()
					.eq(JxglPerformanceRating::getBasicSettingId, bS.getId()));
			// 删除 基础设置
			jxglBasicSettingMapper.delete(new QueryWrapper<JxglBasicSetting>().lambda()
					.eq(JxglBasicSetting::getOrgCode, orgCode));
		}
		
		basicSetting.setOrgCode(orgCode);
		basicSetting.insert();
		if (listPr != null && listPr.size() > 0) {
			listPr.stream().forEach(pr -> pr.setBasicSettingId(basicSetting.getId()));
			int i = jxglPerformanceRatingMapper.insertList(listPr);
		}

		return ResultUtil.success();

	}
	
	/**
	 * 查询-基础设置
	 */
	@GetMapping(value = "/select_basic_setting")
	@ApiOperation(value = "2.查询-基础设置", httpMethod = "GET", notes = "查询-基础设置")
	@ApiOperationSupport(order = 2)
	public Result<Object> selectAT(@CurrentUser UserBean userBean){
		
		JxglBasicSetting bS = jxglBasicSettingMapper.selectAll(userBean.getOrgCode());

		return ResultUtil.data(bS);
		
	}
	
	
	
	//TODO 考核模板
	/**
	 * 新增或编辑-考核模板
	 */
	@PostMapping(value = "/save_appraisal_t")
	@ApiOperation(value = "3.新增或编辑-考核模板", httpMethod = "POST", notes = "新增或编辑-考核模板")
	@ApiOperationSupport(order = 3)
	public Result<Object> saveAtg(@CurrentUser UserBean userBean ,@Validated @RequestBody JxglAppraisalT jxglAppraisalT) throws Exception{
		Integer orgCode = userBean.getOrgCode();
		Integer count = new LambdaQueryChainWrapper<QyzxEmpEntAsso>(qyzxEmpEntAssoMapper)
				.eq(QyzxEmpEntAsso::getEmpNum, userBean.getEmpNum())
				.in(QyzxEmpEntAsso::getUserType, CommonEnum.U_TYPE_ADMIN.getType(),CommonEnum.U_TYPE_C_ADMIN.getType())
				.count();
		
		if (count <= 0) {
			return ResultUtil.error("无权限编辑");
		}
		
		if (jxglAppraisalT.getId() != null && jxglAppraisalT.selectCount(new QueryWrapper<JxglAppraisalT>().lambda()
				.eq(JxglAppraisalT::getId, jxglAppraisalT.getId())) > 0) {
			jxglAppraisalT.updateById();
		}else {
			jxglAppraisalT.setOrgCode(orgCode);
			jxglAppraisalT.insert();
		}
		LambdaQueryWrapper<JxglAppraisalIndicatorsT> q = new QueryWrapper<JxglAppraisalIndicatorsT>().lambda()
		.eq(JxglAppraisalIndicatorsT::getAppraisalTId, jxglAppraisalT.getId());
		
		List<JxglAppraisalIndicatorsT> listAIT = jxglAppraisalIndicatorsTMapper.selectList(q.select(JxglAppraisalIndicatorsT::getId));
		
		List<Integer> listAITId = listAIT != null && listAIT.size() > 0 ? listAIT.stream().map(JxglAppraisalIndicatorsT::getId).collect(Collectors.toList()) : null;
		
		// 删除 原 考核指标模板
		jxglAppraisalIndicatorsTMapper.delete(q);
		// 删除 原 考核项模板
		if (listAITId != null) {
			jsxglAppraisalItemTMapper.delete(new QueryWrapper<JxglAppraisalItemT>().lambda()
					.in(JxglAppraisalItemT::getAppraisalIndicatorsTId, listAITId));
		}
		
		List<JxglAppraisalIndicatorsT> aITs = jxglAppraisalT.getAppraisalIndicatorsTs();
		for (JxglAppraisalIndicatorsT aIT : aITs) {
			aIT.setAppraisalTId(jxglAppraisalT.getId());
			// 新增 考核指标模板
			if (aIT.insert()) {
				List<JxglAppraisalItemT> aItemTs = aIT.getAppraisalItemTs();
				if (aItemTs != null && aItemTs.size() > 0) {
					// 批量新增 考核项模板
					aItemTs.stream().forEach(aItemT -> aItemT.setAppraisalIndicatorsTId(aIT.getId()));
					int c = jsxglAppraisalItemTMapper.insertList(aItemTs);
				}
			}else {
				throw new Exception();
			}
			
		}

		return ResultUtil.success();
	}
	
	
	/**
	 * 查询-考核模板-列表
	 */
	@GetMapping(value = "/selects_appraisal_t")
	@ApiOperation(value = "4.查询-考核模板-列表", httpMethod = "GET", notes = "查询-考核模板-列表")
	@ApiOperationSupport(order = 4)
	public Result<Object> selectListAT(@CurrentUser UserBean userBean){
		
		List<JxglAppraisalT> listAT = jxglAppraisalTMapper.selectList(new QueryWrapper<JxglAppraisalT>().lambda()
				.eq(JxglAppraisalT::getOrgCode, userBean.getOrgCode()));
		
		return ResultUtil.data(listAT);
		
	}
	
	/**
	 * 查询-考核模板-详情
	 */
	@GetMapping(value = "/selects_appraisal_t/{id}")
	@ApiOperation(value = "5.查询-考核模板-详情", httpMethod = "GET", notes = "查询-考核模板-详情")
	@ApiOperationSupport(order = 5)
	public Result<Object> selectAT(@CurrentUser UserBean userBean,@PathVariable Integer id){
		
		JxglAppraisalT aT = jxglAppraisalTMapper.selectDetailById(id);

		return ResultUtil.data(aT);
		
	}
	
	/**
	 * 删除-考核模板
	 */
	@DeleteMapping(value = "/del_appraisal_t/{id}")
	@ApiOperation(value = "6.删除-考核模板", httpMethod = "DELETE", notes = "删除-考核模板")
	@ApiOperationSupport(order = 6)
	public Result<Object> delAT(@CurrentUser UserBean userBean,@PathVariable Integer id) throws Exception{
		
		Integer orgCode = userBean.getOrgCode();
		Integer count = new LambdaQueryChainWrapper<QyzxEmpEntAsso>(qyzxEmpEntAssoMapper)
				.eq(QyzxEmpEntAsso::getEmpNum, userBean.getEmpNum())
				.in(QyzxEmpEntAsso::getUserType, CommonEnum.U_TYPE_ADMIN.getType(),CommonEnum.U_TYPE_C_ADMIN.getType())
				.count();
		
		if (count <= 0) {
			return ResultUtil.error("无权限编辑");
		}
		
		jxglAppraisalTMapper.delete(new QueryWrapper<JxglAppraisalT>().lambda()
				.eq(JxglAppraisalT::getId, id)
				.eq(JxglAppraisalT::getOrgCode, orgCode));

		return ResultUtil.success();
		
	}
	
	/**
	 * 新增-绩效考核
	 */
	@PostMapping(value = "/save_performance_appraisal")
	@ApiOperation(value = "7.新增-绩效考核", httpMethod = "POST", notes = "新增-绩效考核")
	@ApiOperationSupport(order = 7)
	public Result<Object> savePA(@CurrentUser UserBean userBean,@Validated @RequestBody PerformanceAppraisalDto performanceAppraisalDto) throws Exception{
		
		JxglPerformanceAppraisal pA = JxglPerformanceAppraisal.builder().build();
		BeanUtil.copyProperties(performanceAppraisalDto, pA, "processNode","beingAppraisalPerson","appraisalTId");
		pA.setProcessNode(ObjectUtil.serialize(performanceAppraisalDto.getProcessNode()));
		pA.setBeingAppraisalPerson(ObjectUtil.serialize(performanceAppraisalDto.getBeingAppraisalPerson()));
		pA.insert();
		
		List<ProcessNode> listPN = performanceAppraisalDto.getProcessNode();
		List<BeingAppraisalPerson> listBAP = performanceAppraisalDto.getBeingAppraisalPerson();
		// 被考核人员
		List<BeingAppraisalPerson> listEmp = listBAP.stream()
			.filter(bAP -> 
				BeingAppraisalType.EMPLOYEE.getType().equals(bAP.getType()) && 
				BeingAppraisalSts.NEED.getType().equals(bAP.getType()))
			.collect(Collectors.toList());

		// 被考核部门
		List<BeingAppraisalPerson> listDept = listBAP.stream()
				.filter(bAP -> 
					BeingAppraisalType.DEPARTMENT.getType().equals(bAP.getType()) && 
					!BeingAppraisalSts.NEED.getType().equals(bAP.getType()))
				.collect(Collectors.toList());
		
		// 无需被考核人
		List<BeingAppraisalPerson> listNotEmp = listBAP.stream()
				.filter(bAP -> 
					BeingAppraisalType.EMPLOYEE.getType().equals(bAP.getType()) && 
					!BeingAppraisalSts.NOT_NEED.getType().equals(bAP.getType()))
				.collect(Collectors.toList());
		
		return ResultUtil.success();
		
	}
}
